package com.caimei365.user.model.po;

import java.math.BigDecimal;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import java.io.Serializable;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.ibatis.type.Alias;
import com.fasterxml.jackson.annotation.JsonFormat;
import java.util.Date;

/**
 * 用户信息对象 user
 *
 * @author Kaick
 * @date 2023-09-25
 */
@Accessors(fluent  = true )
@Data
@Alias("CmUser")
public class CmUser implements Serializable
{
    private static final long serialVersionUID = 1L;

    /** $column.columnComment */
    private String userID;

    /** 采美组织默认为0,具体对应cm_mall_organize表ID */
    private String userOrganizeID;

    /** 企业账号名【供应商，目前只存数据不能用于登录】 */
    private String account;

    /** 个人手机号码（协销） */
    private String mobile;

    /** 企业绑定手机号（机构，供应商） */
    private String bindMobile;

    /** 用户权限 0游客 1 普通用户 2 会员机构 3 供应商 4 协销 5 普通机构 6 颜选美学商城用户【V6.2.0版本后0和1不存在】 */
    private Integer userPermission;

    /** 用户身份 0、个人 1、协销 2、会员机构 3、供应商 4.普通机构 6、颜选美学商城用户【V6.2.0版本后0不存在】 */
    private String userIdentity;

    /** 邮箱 */
    private String email;

    /** 用户名（机构联系人，供应商简称） */
    private String userName;

    /** 真实姓名（供应商的联系人，机构不使用） */
    private String realName;

    /** 注册来源: 0网站 1小程序 */
    private String source;

    /** 头像 */
    private String image;

    /** 密码 */
    private String password;

    /** 名称（机构名称，供应商的公司名称） */
    private String name;

    /** 见枚举UserType(1供应商,2协销经理,32协销,3会员机构,4普通机构,6颜选美学商城) */
    private String registerUserTypeID;

    /** 供应商状态， 3待审核， 90已上线，91已下线，92审核不通过 */
    private Integer manufacturerStatus;

    /** 供应商Id */
    private Integer shopID;

    /** 审核状态 */
    private String auditStatus;

    /** 审核时间 */
    private String auditTime;

    /** 审核备注 */
    private String auditNote;

    /** 注册时间 */
    private String registerTime;

    /** 注册ip */
    private String registerIP;

    /** 注册IP所属地 */
    private String ipAddress;

    /** 登录时间 */
    private String loginTime;

    /** 登录ip */
    private String loginIP;

    /** 用户状态，1正常，0冻结 */
    private String validFlag;

    /** 会所状态，见表c_clubstatus或枚举ClubStatus */
    private Integer clubStatus;

    /** 会所Id */
    private Integer clubID;

    /** 同意协议标志 */
    private String agreeFlag;

    /** 创客状态 */
    private Integer serviceProviderStatus;

    /** 创客Id */
    private Integer serviceProviderID;

    /** 账户线下余额 */
    private BigDecimal userMoney;

    /** 账户实际可用余额（提交订单未支付的被抵扣后的余额） */
    private BigDecimal ableUserMoney;

    /** 退出时间 */
    private String logoffTime;

    /** $column.columnComment */
    private String appKey;

    /** $column.columnComment */
    private String appSecret;

    /** 扫描标志(4 CRM拉上来的会所) 0待扫描 1 已扫描 2已上线 */
    private Integer scanFlag;

    /** 采美豆数量 */
    private Integer userBeans;

    /** 是否已经引导过(供应商首次登陆操作引导/普通机构升级引导) */
    private Integer guideFlag;

    /** $column.columnComment */
    private Integer loginFailTime;

    /** $column.columnComment */
    private String tipStatus;

    /** 账户线上余额 */
    private BigDecimal onlineMoney;

    public String getUserID() {
        return userID;
    }

    public void setUserID(String userID) {
        this.userID = userID;
    }

    public String getUserOrganizeID() {
        return userOrganizeID;
    }

    public void setUserOrganizeID(String userOrganizeID) {
        this.userOrganizeID = userOrganizeID;
    }

    public String getAccount() {
        return account;
    }

    public void setAccount(String account) {
        this.account = account;
    }

    public String getMobile() {
        return mobile;
    }

    public void setMobile(String mobile) {
        this.mobile = mobile;
    }

    public String getBindMobile() {
        return bindMobile;
    }

    public void setBindMobile(String bindMobile) {
        this.bindMobile = bindMobile;
    }

    public Integer getUserPermission() {
        return userPermission;
    }

    public void setUserPermission(Integer userPermission) {
        this.userPermission = userPermission;
    }

    public String getUserIdentity() {
        return userIdentity;
    }

    public void setUserIdentity(String userIdentity) {
        this.userIdentity = userIdentity;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public String getRealName() {
        return realName;
    }

    public void setRealName(String realName) {
        this.realName = realName;
    }

    public String getSource() {
        return source;
    }

    public void setSource(String source) {
        this.source = source;
    }

    public String getImage() {
        return image;
    }

    public void setImage(String image) {
        this.image = image;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getRegisterUserTypeID() {
        return registerUserTypeID;
    }

    public void setRegisterUserTypeID(String registerUserTypeID) {
        this.registerUserTypeID = registerUserTypeID;
    }

    public Integer getManufacturerStatus() {
        return manufacturerStatus;
    }

    public void setManufacturerStatus(Integer manufacturerStatus) {
        this.manufacturerStatus = manufacturerStatus;
    }

    public Integer getShopID() {
        return shopID;
    }

    public void setShopID(Integer shopID) {
        this.shopID = shopID;
    }

    public String getAuditStatus() {
        return auditStatus;
    }

    public void setAuditStatus(String auditStatus) {
        this.auditStatus = auditStatus;
    }

    public String getAuditTime() {
        return auditTime;
    }

    public void setAuditTime(String auditTime) {
        this.auditTime = auditTime;
    }

    public String getAuditNote() {
        return auditNote;
    }

    public void setAuditNote(String auditNote) {
        this.auditNote = auditNote;
    }

    public String getRegisterTime() {
        return registerTime;
    }

    public void setRegisterTime(String registerTime) {
        this.registerTime = registerTime;
    }

    public String getRegisterIP() {
        return registerIP;
    }

    public void setRegisterIP(String registerIP) {
        this.registerIP = registerIP;
    }

    public String getIpAddress() {
        return ipAddress;
    }

    public void setIpAddress(String ipAddress) {
        this.ipAddress = ipAddress;
    }

    public String getLoginTime() {
        return loginTime;
    }

    public void setLoginTime(String loginTime) {
        this.loginTime = loginTime;
    }

    public String getLoginIP() {
        return loginIP;
    }

    public void setLoginIP(String loginIP) {
        this.loginIP = loginIP;
    }

    public String getValidFlag() {
        return validFlag;
    }

    public void setValidFlag(String validFlag) {
        this.validFlag = validFlag;
    }

    public Integer getClubStatus() {
        return clubStatus;
    }

    public void setClubStatus(Integer clubStatus) {
        this.clubStatus = clubStatus;
    }

    public Integer getClubID() {
        return clubID;
    }

    public void setClubID(Integer clubID) {
        this.clubID = clubID;
    }

    public String getAgreeFlag() {
        return agreeFlag;
    }

    public void setAgreeFlag(String agreeFlag) {
        this.agreeFlag = agreeFlag;
    }

    public Integer getServiceProviderStatus() {
        return serviceProviderStatus;
    }

    public void setServiceProviderStatus(Integer serviceProviderStatus) {
        this.serviceProviderStatus = serviceProviderStatus;
    }

    public Integer getServiceProviderID() {
        return serviceProviderID;
    }

    public void setServiceProviderID(Integer serviceProviderID) {
        this.serviceProviderID = serviceProviderID;
    }

    public BigDecimal getUserMoney() {
        return userMoney;
    }

    public void setUserMoney(BigDecimal userMoney) {
        this.userMoney = userMoney;
    }

    public BigDecimal getAbleUserMoney() {
        return ableUserMoney;
    }

    public void setAbleUserMoney(BigDecimal ableUserMoney) {
        this.ableUserMoney = ableUserMoney;
    }

    public String getLogoffTime() {
        return logoffTime;
    }

    public void setLogoffTime(String logoffTime) {
        this.logoffTime = logoffTime;
    }

    public String getAppKey() {
        return appKey;
    }

    public void setAppKey(String appKey) {
        this.appKey = appKey;
    }

    public String getAppSecret() {
        return appSecret;
    }

    public void setAppSecret(String appSecret) {
        this.appSecret = appSecret;
    }

    public Integer getScanFlag() {
        return scanFlag;
    }

    public void setScanFlag(Integer scanFlag) {
        this.scanFlag = scanFlag;
    }

    public Integer getUserBeans() {
        return userBeans;
    }

    public void setUserBeans(Integer userBeans) {
        this.userBeans = userBeans;
    }

    public Integer getGuideFlag() {
        return guideFlag;
    }

    public void setGuideFlag(Integer guideFlag) {
        this.guideFlag = guideFlag;
    }

    public Integer getLoginFailTime() {
        return loginFailTime;
    }

    public void setLoginFailTime(Integer loginFailTime) {
        this.loginFailTime = loginFailTime;
    }

    public String getTipStatus() {
        return tipStatus;
    }

    public void setTipStatus(String tipStatus) {
        this.tipStatus = tipStatus;
    }

    public BigDecimal getOnlineMoney() {
        return onlineMoney;
    }

    public void setOnlineMoney(BigDecimal onlineMoney) {
        this.onlineMoney = onlineMoney;
    }
}

