package com.caimei365.user.service.impl;

import com.caimei365.user.components.RedisService;
import com.caimei365.user.components.WeChatService;
import com.caimei365.user.feign.ToolsFeign;
import com.caimei365.user.mapper.HeHeMapper;
import com.caimei365.user.model.ResponseJson;
import com.caimei365.user.model.dto.HeHeUserDto;
import com.caimei365.user.model.po.CmHeHeUserPo;
import com.caimei365.user.model.po.CouponSharePo;
import com.caimei365.user.model.po.UserPo;
import com.caimei365.user.model.vo.HeHeUserVo;
import com.caimei365.user.service.HeHeService;
import com.caimei365.user.service.RemoteCallService;
import com.caimei365.user.utils.CodeUtil;
import com.caimei365.user.utils.message.InsideMessage;
import com.caimei365.user.utils.message.MessageModel;
import com.caimei365.user.utils.message.MessageType;
import com.caimei365.user.utils.message.MqInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/4/21
 */
@Slf4j
@Service
public class HeHeServiceImpl implements HeHeService {
    @Resource
    private HeHeMapper heHeMapper;
    @Resource
    private WeChatService weChatService;
    @Resource
    private RedisService redisService;
    @Value("${spring.cloud.config.profile}")
    private String active;
    @Resource
    private RemoteCallService remoteCallService;
    @Resource
    private AsyncService asyncService;
    @Resource
    private ToolsFeign toolsFeign;
    @Override
    public ResponseJson<HeHeUserVo> authorization(String code, HttpHeaders headers) {
        // 小程序微信授权获取登录信息
        ResponseJson<Map<String, Object>> appletsInfo = weChatService.getInfoMapByApplets(code, headers, 2);
        if (appletsInfo.getCode() == -1) {
            return ResponseJson.error(appletsInfo.getMsg(), null);
        }
        Map<String, Object> infoData = appletsInfo.getData();
        String openId = (String) infoData.get(WeChatService.Keys.OPEN_ID);
        HeHeUserVo heHeUser = heHeMapper.findHeHeUserByOpenId(openId);
        if (heHeUser == null) {
            heHeUser = new HeHeUserVo();
            heHeUser.setOpenId(openId);
            return ResponseJson.error(-2, "用户未注册", heHeUser);
        }
        asyncService.loginUpdateBeans(heHeUser.getUserId(), null);
        return ResponseJson.success(heHeUser);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseJson<HeHeUserVo> mobileLogin(HeHeUserDto heHeUserDto) {
        Date date = new Date();
        String currentTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(date);
        String redisSmsCode = (String) redisService.get("heCode:" + heHeUserDto.getMobile());
        if (!"prod".equals(active)) {
            // 本地和测试环境默认验证码为666666
            redisSmsCode = "666666";
        }
        if (null != redisSmsCode && !"null".equals(redisSmsCode)) {
            if (redisSmsCode.equals(heHeUserDto.getVerificationCode())) {
                HeHeUserVo heHeUser = heHeMapper.findHeHeUserByMobile(heHeUserDto.getMobile());
                if (heHeUser != null) {
                    heHeUser.setOpenId(heHeUserDto.getOpenId());
                    heHeUser.setHeadImgUrl(heHeUserDto.getHeadImgUrl());
                    heHeUser.setNickName(heHeUserDto.getNickName());
                    heHeMapper.updateHeHeUser(heHeUser);
                } else {
                    UserPo user = new UserPo();
                    user.setBindMobile("H" + heHeUserDto.getMobile());
                    user.setUserName(heHeUserDto.getNickName());
                    user.setName(heHeUserDto.getNickName());
                    user.setPassword("HeHe");
                    //6:颜选美学商城用户
                    user.setUserIdentity(6);
                    user.setRegisterUserTypeId(6);
                    user.setRegisterTime(currentTime);
                    user.setValidFlag(1);
                    heHeMapper.insertUser(user);

                    CmHeHeUserPo heUserPo = new CmHeHeUserPo();
                    BeanUtils.copyProperties(heHeUserDto, heUserPo);
                    heUserPo.setUserId(user.getUserId());
                    heUserPo.setUserIdentity("1");
                    heUserPo.setAddTime(date);
                    heHeMapper.insertHeHeUser(heUserPo);
                    // 查询是否存在新人券
                    List<Integer> newUserCouponIds = heHeMapper.getCurrentCouponIds(3);
                    boolean isBeta = "dev".equals(active) || "beta".equals(active);
                    // 注册成功，推送新人券通知短信
                    newUserCouponIds.forEach(couponId->{
                        // 查询优惠券金额
                        Double couponAmount = heHeMapper.getCouponAmount(couponId);
                        String content = "恭喜您成功注册登录。现赠送" + couponAmount + "元优惠券到您的领券中心，请赶紧登录颜选美学小程序领取下单吧。退订回T";
                        boolean result = remoteCallService.remoteSendSms(17, 3, heHeUserDto.getMobile(), content);
                        log.info("【颜选美学新人券派送】mobile:" + heHeUserDto.getMobile() + ",result:" + result);
                        //站内信
                        MessageModel<InsideMessage> insideMessageMessageModel = new MessageModel<>();
                        insideMessageMessageModel.code(MessageType.WEB_INSIDE_MESSAGE)
                                .mqInfo(new MqInfo().topic("MessageLine").delay(1).async(0))
                                .info(new InsideMessage()
                                        .userType(4)
                                        .messageType(4)
                                        .couponType(4)
                                        .content("恭喜您成功注册登录。现赠送"+couponAmount+"元优惠券到您的领券中心，快去领取下单吧。")
                                        .thisId(heUserPo.getUserId())
                                );
                        toolsFeign.sendCommonMessage(insideMessageMessageModel);
                    });
                    heHeUser = heHeMapper.findHeHeUserByMobile(heHeUserDto.getMobile());
                    Integer shareUserId = heHeUserDto.getShareUserId();
                    if (null != shareUserId && 0 != shareUserId) {
                        Integer receiveUserId = heHeUser.getUserId();
                        CouponSharePo couponSharePo = new CouponSharePo();
                        couponSharePo.setShareUserId(shareUserId);
                        couponSharePo.setReceiveUserId(receiveUserId);
                        List<Integer> couponIds = heHeMapper.getCurrentCouponIds(4);
                        couponIds.forEach(couponId->{
                            couponSharePo.setShareCouponId(couponId);
                            // 保存好友邀请记录
                            heHeMapper.insertCouponShare(couponSharePo);
                            // 查询邀请人手机号
                            String mobile = heHeMapper.findUserMobile(shareUserId);
                            // 查询优惠券金额
                            Double couponAmount = heHeMapper.getCouponAmount(couponId);
                            String content = "恭喜您成功邀请一位好友注册登录，现赠送"+couponAmount+"元优惠券到您的领券中心，请赶紧登录颜选美学小程序领取下单吧。退订回T";
                            boolean result = remoteCallService.remoteSendSms(18, 3, mobile, content);
                            log.info("【颜选美学好友邀请券派送】mobile:" + mobile + ",result:" + result);
                            //站内信
                            MessageModel<InsideMessage> insideMessageMessageModel = new MessageModel<>();
                            insideMessageMessageModel.code(MessageType.WEB_INSIDE_MESSAGE)
                                    .mqInfo(new MqInfo().topic("MessageLine").delay(1).async(0))
                                    .info(new InsideMessage()
                                            .userType(4)
                                            .messageType(4)
                                            .couponType(5)
                                            .content("恭喜您成功邀请一位好友注册登录，现赠送"+couponAmount+"元优惠券到您的领券中心，快去领取下单吧。")
                                            .thisId(shareUserId)
                                    );
                            toolsFeign.sendCommonMessage(insideMessageMessageModel);
                        });
                    }
                }
                return ResponseJson.success(heHeUser);
            }
        }
        return ResponseJson.error("验证码错误", null);
    }

    @Override
    public ResponseJson<String> sendVerificationCode(String mobile) {
        String randomCode = CodeUtil.generateCodeInt(6);
        String content = "您“颜选美学”小程序的验证码为："+randomCode+"，验证码 5 分钟内有效。";
        boolean smsFlag = remoteCallService.remoteSendSms(0, 1, mobile, content);
        if (!smsFlag) {
            // 短信发送失败重试一次
            remoteCallService.remoteSendSms(0, 1, mobile, content);
        }
        log.info("颜选美学商城注册验证码:" + randomCode);
        redisService.set("heCode:" + mobile, randomCode, 300L);
        return ResponseJson.success("发送成功");
    }

    @Override
    public ResponseJson<String> getGroupQrCode() {
        String image = heHeMapper.getGroupQrCode();
        return ResponseJson.success(image);
    }
}
