package com.caimei365.user.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.user.components.RedisService;
import com.caimei365.user.feign.CommodityFeign;
import com.caimei365.user.mapper.*;
import com.caimei365.user.model.ResponseJson;
import com.caimei365.user.model.dto.*;
import com.caimei365.user.model.po.*;
import com.caimei365.user.model.vo.*;
import com.caimei365.user.service.ShopService;
import com.caimei365.user.utils.*;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import redis.clients.jedis.Jedis;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.YearMonth;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static com.alibaba.fastjson.JSON.parseObject;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/3/9
 */
@Slf4j
@Service
public class ShopServiceImpl implements ShopService {
    @Value("${caimei.wwwDomain}")
    private String wwwDomain;
    @Resource
    private RedisService redisService;
    @Resource
    private CommodityFeign commodityFeign;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private ShopMapper shopMapper;
    @Resource
    private LoginMapper loginMapper;
    @Resource
    private ArticleMapper articleMapper;
    @Resource
    private BaikeProductMapper baikeProductMapper;
    @Resource
    private PersonalCenterMapper personalCenterMapper;
    @Value("${spring.cloud.config.profile}")
    private String active;


    /**
     * 供应商快捷登录
     *
     * @param userId
     * @return
     */
    @Override
    public ResponseJson<UserLoginVo> shortcutLogin(Integer userId) {
        UserLoginVo userLoginVo = shopMapper.getShopByUserId(userId);
        if (null != userLoginVo) {
            // 生成token给用户
            String token = JwtUtil.createToken(userId);
            // 为了过期续签，将token存入redis，并设置超时时间
            redisService.set(token, token, JwtUtil.getExpireTime());
            userLoginVo.setToken(token);
            if (null != userLoginVo.getShopId()) {
                if (null != userLoginVo.getShopStatus() && null != userLoginVo.getUserIdentity() && 3 == userLoginVo.getUserIdentity()) {

                    if (3 == userLoginVo.getShopStatus()) {
                        return ResponseJson.error(-1, "您的企业账号正在加速审核中,审核通过后即可登录", userLoginVo);
                    }
                    if (91 == userLoginVo.getShopStatus()) {
                        return ResponseJson.error(-1, "您的企业账号已被冻结，请联系客服处理", userLoginVo);
                    }
                    if (92 == userLoginVo.getShopStatus()) {
                        return ResponseJson.error(-3, "您的企业账号审核未通过", userLoginVo);
                    }
                }
                // 是否可为机构下单
                boolean flag = false;
                // 供应商商品上架平台信息
                List<String> shopProductList = loginMapper.getShopProductGroudMall(userLoginVo.getShopId());
                if (null != shopProductList) {
                    for (String product : shopProductList) {
                        if (StringUtils.isNotBlank(product)) {
                            // 商品上架平台是否在联合丽格上架
                            if (product.contains("4")) {
                                flag = true;
                            }
                        }
                    }
                } else {
                    flag = false;
                }
                userLoginVo.setShopIsOrder(flag);
            } else {
                return ResponseJson.error(-1, "该账号未成为供应商", null);
            }
        } else {
            return ResponseJson.error(-1, "用户数据异常", null);
        }
        return ResponseJson.success("登录成功", userLoginVo);
    }

    /**
     * 根据用户Id查询供应商资料
     *
     * @param userId 用户Id
     * @return Map(userPo, clubPo)
     */
    @Override
    public ResponseJson<Map<String, Object>> getShopUserInfo(Integer userId) {
        if (null == userId) {
            return ResponseJson.error("参数异常：用户Id不能为空！", null);
        }
        // 用户信息
        UserVo user = baseMapper.getUserByUserId(userId);
        if (user == null) {
            return ResponseJson.error("用户信息不存在", null);
        }
        if (!StringUtils.isBlank(user.getAuditNote()) && 2 == user.getAuditStatus()) {
            if (user.getAuditNote().contains(",")) {
                List<String> list = Arrays.asList(user.getAuditNote().split(","));
                user.setAuditNoteList(list);
            } else {
                user.setAuditNoteList(Collections.singletonList(user.getAuditStatus().toString()));
            }
        }
        // 供应商信息
        ShopVo shop = shopMapper.getShopById(user.getShopId());
        if (shop == null) {
            return ResponseJson.error("供应商信息不存在", null);
        }
        if (null != shop.getTownId()) {
            TownVo town = baseMapper.getTown(shop.getTownId());
            CityVo city = baseMapper.getCity(town.getCityId());
            ProvinceVo province = baseMapper.getProvince(city.getProvinceId());
            shop.setCityId(city.getCityId());
            shop.setProvinceId(province.getProvinceId());
            shop.setProvincialAddress(province.getName() + "" + city.getName() + "" + town.getName());
        }
        // 荣誉证书
        List<String> honorCertification = shopMapper.getShopCert(user.getShopId(), 1);
        shop.setHonorCertification(honorCertification);
        // 生产经营证书
        List<String> operationLicence = shopMapper.getShopCert(user.getShopId(), 2);
        if (null != operationLicence && operationLicence.size() > 0) {
            shop.setOperationLicence(operationLicence.get(0));
        }
        // 产品证书
        List<String> productCertification = shopMapper.getShopCert(user.getShopId(), 3);
        shop.setProductCertification(productCertification);
        // 卫生许可证
        List<String> hygienicLicense = shopMapper.getShopCert(user.getShopId(), 5);
        if (null != hygienicLicense && hygienicLicense.size() > 0) {
            shop.setHygienicLicense(hygienicLicense.get(0));
        }
        // 税务登记证
        List<String> taxLicense = shopMapper.getShopCert(user.getShopId(), 6);
        if (null != taxLicense && taxLicense.size() > 0) {
            shop.setTaxLicense(taxLicense.get(0));
        }
        HashMap<String, Object> map = new HashMap(2);
        map.put("user", user);
        map.put("shop", shop);
        return ResponseJson.success(map);
    }


    /**
     * 修改供应商资料
     *
     * @param shopUpdateDto        ShopUpdateDto{
     *                             shopId                    供应商Id
     *                             userId                    用户Id
     *                             name                      组织名称
     *                             shortName                     供应商公司简称(shortName)
     *                             email                     邮箱
     *                             contractPhone             固定电话
     *                             linkMan                   联系人
     *                             provinceId                省(provinceID)
     *                             cityId                    市(cityID)
     *                             townId                    所在县区Id(townID)
     *                             address                   地址
     *                             socialCreditCode          统一社会信用代码(营业执照编号)
     *                             businessLicense           营业执照(businessLicenseImage)
     *                             firstShopType             医疗=1和非医疗=2
     *                             secondShopType            医疗的二级分类 一类器械=1、二类器械 =2、三类器械=3、其他=4 /// 1和非医疗没有二级分类
     *                             mainProduct               主打项目(mainpro)
     *                             mainProductDesc           主打商品说明(productDesc)
     *                             legalPerson               法人代表
     *                             registeredCapital         注册资本
     *                             faxNumber                 传真号(fax)
     *                             companyNature             传真号(nature)
     *                             turnover                  年营业额
     *                             medicalPracticeLicenseImg 医疗执业许可证(medicalPracticeLicenseImg1)
     *                             shopDesc                  公司介绍(info)
     *                             businessScope             经营范围
     *                             logo                      公司LOGO
     *                             }
     * @param operationLicence     生产经营证书(productionLicence)
     * @param hygienicLicense      卫生许可证
     * @param taxLicense           税务登记证
     * @param honorCertification   荣誉证书(certificateHonor)
     * @param productCertification 产品证书
     * @return ClubUpdateDto
     */
    @Override
    public ResponseJson<Void> updateShopUserInfo(ShopUpdateDto shopUpdateDto, String operationLicence, String hygienicLicense, String taxLicense, String honorCertification, String productCertification) {
        // 参数校验
        if (null == shopUpdateDto.getUserId()) {
            return ResponseJson.error("参数异常：用户Id不能为空！");
        }
        if (null == shopUpdateDto.getShopId()) {
            return ResponseJson.error("参数异常：供应商Id不能为空！");
        }
        if (StringUtils.isNotBlank(shopUpdateDto.getContractEmail())) {
            Integer userIdByEmail = baseMapper.getUserIdByEmail(shopUpdateDto.getContractEmail());
            if (null != userIdByEmail && !userIdByEmail.equals(shopUpdateDto.getUserId())) {
                return ResponseJson.error("该邮箱已被使用");
            }
        }
        /*
            组装用户数据 user
         */
        UserPo user = new UserPo();
        user.setUserId(shopUpdateDto.getUserId());
        user.setName(shopUpdateDto.getName());
        user.setUserName(shopUpdateDto.getShortName());
        user.setEmail(shopUpdateDto.getContractEmail());
        // 更新供应商用户信息
        shopMapper.updateShopUserByUpdateInfo(user);
        // 更新供应商信息
        shopMapper.updateShopByUpdateInfo(shopUpdateDto);
        if (StringUtils.isNotBlank(shopUpdateDto.getSocialCreditCode())) {
            // 清除数据库证书数据
            shopMapper.deleteShopCertByShopId(shopUpdateDto.getShopId());
            ShopCertPo shopCert = new ShopCertPo();
            shopCert.setShopId(shopUpdateDto.getShopId());
            shopCert.setShopCertTypeId(2);
            shopCert.setImage(operationLicence);
            shopCert.setName("生产经营证书");
            // 保存生产经营证书
            shopMapper.insertShopCert(shopCert);
            shopCert.setShopCertTypeId(5);
            shopCert.setImage(hygienicLicense);
            shopCert.setName("卫生许可证");
            // 保存卫生许可证
            shopMapper.insertShopCert(shopCert);
            shopCert.setShopCertTypeId(6);
            shopCert.setImage(taxLicense);
            shopCert.setName("税务登记证");
            // 保存税务登记证
            shopMapper.insertShopCert(shopCert);
            if (StringUtils.isNotBlank(honorCertification)) {
                String[] images = honorCertification.split(",");
                for (String image : images) {
                    if (StringUtils.isNotBlank(image)) {
                        shopCert.setShopCertTypeId(1);
                        shopCert.setImage(image);
                        shopCert.setName("荣誉证书");
                        // 保存荣誉证书
                        shopMapper.insertShopCert(shopCert);
                    }
                }
            }
            if (StringUtils.isNotBlank(productCertification)) {
                String[] images = productCertification.split(",");
                for (String image : images) {
                    if (StringUtils.isNotBlank(image)) {
                        shopCert.setShopCertTypeId(3);
                        shopCert.setImage(image);
                        shopCert.setName("产品证书");
                        // 保存产品证书
                        shopMapper.insertShopCert(shopCert);
                    }
                }
            }
        }
        return ResponseJson.success("修改供应商资料成功", null);
    }

    /**
     * 供应商首页数据
     *
     * @param shopId 供应商Id
     */
    @Override
    public ResponseJson<ShopHomeVo> getShopHomeData(Integer shopId, Integer userId, Integer userIdentity) {
        /** 薇旖美特殊需求，非福建服务商名下的机构无法查看其主页
         *  1.供应商自己可以看
         *  2.薇旖美服务商+名下协销和机构
         */
//        if (shopId == 10131) {
//            boolean b = true;
//            // 协销
//            if (1 == userIdentity) {
//
//            }
//            // 供应商
//            else if (3 == userIdentity) {
//                b = 22965 != userId;
//            }
//            // 机构
//            else if (2 == userIdentity || 4 == userIdentity) {
//                // 根据服务商（协销组长userId）查询名下小组内协销匹配机构
//                Integer spId = shopMapper.findSpId(userId);
//                spId = null == spId ? 0 : spId;
//
//            }
//            if(b){
//                return ResponseJson.error(-2, "无权限");
//            }
//        }
        ShopHomeVo supplier = shopMapper.getShopHomeData(shopId);
        if (supplier != null && supplier.getBusinessScope() != null) {
            String[] businessScope = supplier.getBusinessScope().split("/");
            supplier.setBusinessScopeArr(businessScope);
            supplier.setLogo(ImageUtils.getImageURL("shopLogo", supplier.getLogo(), 0, wwwDomain));
            supplier.setMedicalPracticeLicense(shopMapper.medicalPracticeLicense(shopId));
        }
        //公司资质照片
        List<String> productionLicence = shopMapper.getShopCertById(shopId, 2);
        if (null != productionLicence && productionLicence.size() > 0) {
            supplier.setMedicalPracticeLicenseImg1(productionLicence.get(0));
        }
        List<String> hygienicLicense = shopMapper.getShopCertById(shopId, 5);
        if (null != hygienicLicense && hygienicLicense.size() > 0) {
            supplier.setMedicalPracticeLicenseImg2(hygienicLicense.get(0));
        }
        List<String> taxLicense = shopMapper.getShopCertById(shopId, 6);
        if (null != taxLicense && taxLicense.size() > 0) {
            supplier.setMedicalPracticeLicenseImg3(taxLicense.get(0));
        }
        // banner
        List<ShopBannerVo> shopHomeImages = shopMapper.getShopHomeImages(shopId, null);
        supplier.setShopHomeImages(shopHomeImages);
        // 商品类别
        List<CmShopCategoryVo> shopCategory = shopMapper.getShopCategory(shopId, null);
        supplier.setShopCategory(shopCategory);
        return ResponseJson.success(supplier);
    }

    /**
     * 供应商主页列表配置列表
     *
     * @param shopId
     * @param category
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<CmShopCategoryVo>> getShopCategory(Integer shopId, String category, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CmShopCategoryVo> shopCategory = shopMapper.getShopCategory(shopId, category);
        PaginationVo<CmShopCategoryVo> page = new PaginationVo<>(shopCategory);
        return ResponseJson.success(page);
    }

    /**
     * 一键排序
     *
     * @param id
     * @param sort
     * @return
     */
    @Override
    public ResponseJson renewShopCategory(Integer id, Integer sort) {
        shopMapper.updateCategorySort(id, sort);
        return ResponseJson.success();
    }

    /**
     * 供应商商品类别回显
     *
     * @param id
     * @return
     */
    @Override
    public ResponseJson<CmShopCategoryVo> echoShopCategory(Integer id) {
        CmShopCategoryVo shopCategoryById = shopMapper.getShopCategoryById(id);
        List<ProductItemVo> categoryProductList = shopMapper.getCategoryProductById(id);
        categoryProductList.forEach(product -> {
            product.setImage(ImageUtils.getImageURL("product", product.getImage(), 0, wwwDomain));
        });
        shopCategoryById.setProducts(categoryProductList);
        return ResponseJson.success(shopCategoryById);
    }

    /**
     * 选择商品
     *
     * @param shopId
     * @param categoryId
     * @param name
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PaginationVo<ProductItemVo>> getShopProductList(Integer shopId, Integer categoryId, String name, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<ProductItemVo> shopProductList = shopMapper.getShopProductList(shopId, name);
        // 供应商已选择商品, 同类别不可重复选择
        List<Integer> categoryProductIdList = new ArrayList<>();
        if (null != categoryId) {
            categoryProductIdList = shopMapper.getCategoryProductIdList(shopId, categoryId);
        }
        for (ProductItemVo product : shopProductList) {
            product.setImage(ImageUtils.getImageURL("product", product.getImage(), 0, wwwDomain));
            if (null != categoryProductIdList && categoryProductIdList.contains(product.getProductId())) {
                product.setFlag(false);
            } else {
                product.setFlag(true);
            }
        }
        PaginationVo<ProductItemVo> page = new PaginationVo<>(shopProductList);
        return ResponseJson.success(page);
    }

    /**
     * 保存商品类别
     *
     * @param jsonParamsDto
     * @return
     */
    @Override
    public ResponseJson saveShopCategory(JsonParamsDto jsonParamsDto) {
        try {
            // 处理json数据
            JSONObject jsonObject = parseObject(jsonParamsDto.getParams());
            Integer id = jsonObject.getInteger("id");
            Integer shopId = jsonObject.getInteger("shopId");
            String category = jsonObject.getString("category");
            Integer sort = jsonObject.getInteger("sort");
            String productIds = jsonObject.getString("productIds");
            if (null == shopId) {
                return ResponseJson.error(-1, "供应商Id不能为空", null);
            }
            if (StringUtils.isBlank(category) || StringUtils.isEmpty(category)) {
                return ResponseJson.error(-1, "类别名称不能为空", null);
            }
            if (null == sort) {
                return ResponseJson.error(-1, "类别排序不能为空", null);
            }
            if (StringUtils.isBlank(productIds) || StringUtils.isEmpty(productIds)) {
                return ResponseJson.error(-1, "商品不能为空", null);
            }
            CmShopCategoryVo cmShopCategoryVo = new CmShopCategoryVo();
            cmShopCategoryVo.setId(id);
            cmShopCategoryVo.setShopId(shopId);
            cmShopCategoryVo.setCategory(category);
            cmShopCategoryVo.setSort(sort);
            cmShopCategoryVo.setProductIds(productIds);
            if (null == id) {
                // 新增
                shopMapper.insertCategory(cmShopCategoryVo);
            } else {
                // 修改
                shopMapper.updateCategory(cmShopCategoryVo);
            }
            Integer categoryId = cmShopCategoryVo.getId();

            if (StringUtils.isNotBlank(cmShopCategoryVo.getProductIds())) {
                // 处理类别商品
                List<Integer> productIdList = new ArrayList<>();
                // 该类别下所有商品
                List<Integer> categoryProductIds = shopMapper.getCategoryProductIds(categoryId);
                if (cmShopCategoryVo.getProductIds().contains(",")) {
                    String[] productIdArr = cmShopCategoryVo.getProductIds().split(",");
                    for (String productId : productIdArr) {
                        int proId = Integer.parseInt(productId);
                        productIdList.add(proId);
                        if (!categoryProductIds.contains(proId)) {
                            shopMapper.insertCategoryProduct(categoryId, proId);
                        }
                    }
                } else {
                    int proId = Integer.parseInt(cmShopCategoryVo.getProductIds());
                    productIdList.add(proId);
                    if (!categoryProductIds.contains(proId)) {
                        shopMapper.insertCategoryProduct(categoryId, proId);
                    }
                }
                // 排除不在该商品类别的商品
                if (productIdList.size() > 0) {
                    shopMapper.updateCategoryProduct(categoryId, productIdList);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResponseJson.success();
    }

    /**
     * 删除供应商商品类别
     *
     * @param id
     * @return
     */
    @Override
    public ResponseJson deleteCategory(Integer id) {
        shopMapper.deleteCategory(id);
        return ResponseJson.success();
    }

    /**
     * 供应商首页-轮播图片
     *
     * @param shopId 供应商Id
     * @param title  标题
     */
    @Override
    public ResponseJson<PaginationVo<ShopBannerVo>> getShopHomeImages(Integer shopId, String title, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<ShopBannerVo> images = shopMapper.getShopHomeImages(shopId, title);
        PaginationVo<ShopBannerVo> page = new PaginationVo<>(images);
        return ResponseJson.success(page);
    }

    /**
     * 一键排序
     *
     * @param id
     * @param sort
     * @return
     */
    @Override
    public ResponseJson updateShopBannerSort(Integer id, Integer sort) {
        shopMapper.updateShopBannerSort(id, sort);
        return ResponseJson.success();
    }

    /**
     * 供应商-更新轮播图片
     *
     * @param shopBannerDto {
     *                      id      图片id
     *                      shopId  供应商Id
     *                      image   图片
     *                      title   名称
     *                      link    链接
     *                      info    描述
     *                      }
     */
    @Override
    public ResponseJson<Void> saveShopHomeImages(ShopBannerDto shopBannerDto) {
        ShopBannerVo shopBanner = new ShopBannerVo();
        BeanUtils.copyProperties(shopBannerDto, shopBanner);
        if (null != shopBannerDto.getId()) {
            // 更新
            shopBanner.setId(shopBannerDto.getId());
            shopBanner.setSort(shopBanner.getSort());
            shopMapper.updateShopHomeImage(shopBanner);
        } else {
            // 新增
            /*int count = shopMapper.getShopHomeImageCount(shopBannerDto.getShopId());
            if (count >= 4) {
                return ResponseJson.error("主页广告图的上限是4张图！", null);
            }*/
            shopMapper.insertShopHomeImage(shopBanner);
        }
        return ResponseJson.success(null);
    }

    /**
     * 供应商-删除轮播图片
     *
     * @param id 图片id
     */
    @Override
    public ResponseJson<Void> deleteShopHomeImages(Integer id) {
        shopMapper.deleteShopHomeImage(id);
        return ResponseJson.success(null);
    }

    /**
     * 供应商个人中心数据
     *
     * @param userId 用户Id
     */
    @Override
    public ResponseJson<Map<String, Object>> getShopPersonalData(Integer userId, Integer productOrganize) {
        Map<String, Object> result = new HashMap<>(11);
        // 1.用户信息
        UserVo user = baseMapper.getUserByUserId(userId);
        if (user == null) {
            return ResponseJson.error("用户信息不存在", null);
        }
        result.put("user", user);
        // 2.供应商信息
        ShopVo shop = shopMapper.getShopById(user.getShopId());
        if (shop == null) {
            return ResponseJson.error("供应商信息不存在", null);
        }
        shop.setLogo(ImageUtils.getImageURL("shopLogo", shop.getLogo(), 0, wwwDomain));
        result.put("shop", shop);
        // 3.资料完整度
        List<String> productCertification = shopMapper.getShopCertById(user.getShopId(), 3);
        List<String> certificateHonor = shopMapper.getShopCertById(user.getShopId(), 1);
        // 4.是否有荣誉证书与产品证书
        boolean b = productCertification != null && productCertification.size() > 0 && certificateHonor != null && certificateHonor.size() > 0;
        if (StringUtils.isNotBlank(user.getEmail()) && StringUtils.isNotBlank(shop.getLegalPerson()) && StringUtils.isNotBlank(shop.getShopDesc())
                && StringUtils.isNotBlank(shop.getLogo()) && b) {
            result.put("dataIntegrity", "100%");
        } else if (StringUtils.isNotBlank(shop.getLegalPerson()) && StringUtils.isNotBlank(shop.getShopDesc()) && StringUtils.isNotBlank(shop.getLogo())) {
            result.put("dataIntegrity", "90%");
        } else {
            result.put("dataIntegrity", "70%");
        }
        // 5.店铺促销活动
        PromotionsVo promotions = shopMapper.getShopPromotionsByShopId(user.getShopId());
        if (null != promotions && 3 == promotions.getMode()) {
            List<ProductItemVo> productGifts = shopMapper.getPromotionsGifts(promotions.getId());
            productGifts.forEach(p -> {
                p.setImage(ImageUtils.getImageURL("product", p.getImage(), 0, wwwDomain));
            });
            promotions.setGiftList(productGifts);
        }
        result.put("promotions", promotions);
        // 6.未读消息数量
        Integer unReadMessageCount = personalCenterMapper.getUnReadMessageCount(userId);
        result.put("unReadMessageCount", unReadMessageCount);
        // 7.商品数量统计,validFlag 商品状态，见ProductStatus，0逻辑删除 1待审核 2已上架 3已下架 8审核未通过 9已冻结'
        productOrganize = null == productOrganize ? 0 : productOrganize;
        Integer allNum = shopMapper.getShopProductCount(user.getShopId(), productOrganize, null);
        result.put("allNum", allNum);
        // 8.已上架数量
        Integer upNum = shopMapper.getShopProductCount(user.getShopId(), productOrganize, 2);
        result.put("upNum", upNum);
        // 9.已下架数量
        Integer downNum = shopMapper.getShopProductCount(user.getShopId(), productOrganize, 3);
        result.put("downNum", downNum);
        Jedis jedis = new Jedis("172.31.165.27", 6379);
        jedis.auth("6#xsI%b4o@5c3RoE");
        jedis.lpush("constraint", "1");
        result.put("constraint", jedis.rpop("constraint"));

        // 10.单品销售排名
        List<Map<String, Object>> salesRankingList = shopMapper.getSalesRankingList(user.getShopId(), productOrganize);
        if (salesRankingList != null && salesRankingList.size() > 0) {
            salesRankingList.forEach(map -> {
                ProductItemVo product = shopMapper.getProductShown((Integer) map.get("productId"));
                if (product != null) {
                    map.put("name", product.getName());
                    map.put("image", ImageUtils.getImageURL("product", product.getImage(), 0, wwwDomain));
                }
            });
        }
        result.put("salesRankingList", salesRankingList);
        // 11.销售情况统计
        List<Map<String, Object>> salesStatisticsList = new ArrayList<>();
        Date endDayOfDay = DateUtil.getDayEnd();
        // 近一个月销售情况统计
        Date beginDayOfYesterday = DateUtil.addMonth(endDayOfDay, -1);
        Map<String, Object> m1 = shopMapper.getSalesStatistics(user.getShopId(), productOrganize, DateUtil.formatDateTime(beginDayOfYesterday), DateUtil.formatDateTime(endDayOfDay));
        m1.put("time", "近一月");
        salesStatisticsList.add(m1);
        // 近三个月销售情况统计
        Date beginDayOfLastWeek = DateUtil.addMonth(endDayOfDay, -3);
        Map<String, Object> m2 = shopMapper.getSalesStatistics(user.getShopId(), productOrganize, DateUtil.formatDateTime(beginDayOfLastWeek), DateUtil.formatDateTime(endDayOfDay));
        m2.put("time", "近三月");
        salesStatisticsList.add(m2);
        // 近半年销售情况统计
        Date beginDayOfLastMonth = DateUtil.addMonth(endDayOfDay, -6);
        Map<String, Object> m3 = shopMapper.getSalesStatistics(user.getShopId(), productOrganize, DateUtil.formatDateTime(beginDayOfLastMonth), DateUtil.formatDateTime(endDayOfDay));
        m3.put("time", "近半年");
        salesStatisticsList.add(m3);
        // 近一年销售情况统计
        Date beginDayOfLastYear = DateUtil.addMonth(endDayOfDay, -12);
        Map<String, Object> m4 = shopMapper.getSalesStatistics(user.getShopId(), productOrganize, DateUtil.formatDateTime(beginDayOfLastYear), DateUtil.formatDateTime(endDayOfDay));
        m4.put("time", "近一年");
        salesStatisticsList.add(m4);
        result.put("salesStatisticsList", salesStatisticsList);
        // 是否为营销供应商
        Integer shopIsMark = shopMapper.getShopIsMark(shop.getShopId());
        if (null != shopIsMark) {
            result.put("isMark", true);
        } else {
            result.put("isMark", false);
        }
        return ResponseJson.success(result);
    }

    /**
     * constraint加入缓存
     */
    @Scheduled(cron = "0 0 23 * * ?")
    public void Open() {
        Jedis jedis = new Jedis("172.31.165.27", 6379);
        jedis.auth("6#xsI%b4o@5c3RoE");
        jedis.lpush("constraint", "");
    }

    @Override
    public ResponseJson<Map<String, Object>> getShopArticleById(Integer articleId) {
        Map<String, Object> map = new HashMap<>(2);
        // 供应商文章
        ShopArticleVo shopArticle = articleMapper.getShopArticleById(articleId);
        // 文章分类列表
        List<ArticleTypeVo> articleTypeList = articleMapper.getArticleTypeList("0");
        infoTypes(articleTypeList);
        // 文章标签列表
        String articleLabels = articleMapper.getArticleLabelList();
        // 文章关联的标签库
        if (StringUtils.isNotBlank(shopArticle.getLabelIds())) {
            String labelIds = shopArticle.getLabelIds();
            List<String> labels = baseMapper.findLabelsByLableIds(labelIds);
            shopArticle.setLabels(labels);
        }
        map.put("shopArticle", shopArticle);
        map.put("articleTypeList", articleTypeList);
        map.put("articleLabels", articleLabels);
        return ResponseJson.success(map);
    }

    @Override
    public ResponseJson<Void> saveShopArticle(ShopArticleDto shopArticleDto) {
        // 参数校验
        if (null == shopArticleDto.getShopId()) {
            return ResponseJson.error("参数异常，供应商id不能为空", null);
        }
        if (StringUtils.isEmpty(shopArticleDto.getTitle())) {
            return ResponseJson.error("参数异常，文章标题不能为空", null);
        }
        if (StringUtils.isEmpty(shopArticleDto.getLabel())) {
            return ResponseJson.error("参数异常，文章标签不能为空", null);
        }
        if (StringUtils.isEmpty(shopArticleDto.getKeyword())) {
            return ResponseJson.error("参数异常，SEO关键词不能为空", null);
        }
        if (StringUtils.isEmpty(shopArticleDto.getPublisher())) {
            return ResponseJson.error("参数异常，发布人不能为空", null);
        }
        if (StringUtils.isEmpty(shopArticleDto.getRecommendContent())) {
            return ResponseJson.error("参数异常，推荐语不能为空", null);
        }
        if (StringUtils.isEmpty(shopArticleDto.getArticleContent())) {
            return ResponseJson.error("参数异常，文章内容不能为空", null);
        }
        if (null == shopArticleDto.getTypeId()) {
            return ResponseJson.error("参数异常，文章分类不能为空", null);
        }
        if (StringUtils.isEmpty(shopArticleDto.getGuidanceImage())) {
            return ResponseJson.error("参数异常，引导图不能为空", null);
        }
        if (null == shopArticleDto.getStatus()) {
            return ResponseJson.error("参数异常，状态不能为空", null);
        }
        // 保存新增的文章标签
        String articleLabel = shopArticleDto.getLabel();
        String[] labelArr = articleLabel.split(",");
        for (String label : labelArr) {
            Integer dbLabelId = articleMapper.getArticleLabelId(label);
            if (null == dbLabelId) {
                articleMapper.insertArticleLabel(label);
            }
        }
        ArticlePo articlePo = new ArticlePo();
        BeanUtils.copyProperties(shopArticleDto, articlePo);
        // 新增文章标识
        boolean newArticleFlag = null == articlePo.getArticleId() || 0 == articlePo.getArticleId();
        // 保存供应商文章
        if (newArticleFlag) {
            articlePo.setPublishSource(2);
            articlePo.setRecommendStatus(0);
            articlePo.setBasePraise(0);
            articlePo.setBasePv(0);
            articlePo.setPriorityIndex(0);
            articlePo.setAuditStatus(1);
            articlePo.setOnlineStatus(1);
            articleMapper.insertShopArticle(articlePo);
        } else {
            articlePo.setAuditStatus(1);
            articlePo.setOnlineStatus(1);
            articleMapper.updateShopArticle(articlePo);
        }
        // 更新文章索引
        commodityFeign.updateArticleIndex(articlePo.getArticleId());
        if (newArticleFlag) {
            // 新增文章浏览/点赞记录
            articleMapper.insertArticlePraise(articlePo.getArticleId());
            // 新增文章 百度链接实时推送
            if ("prod".equals(active)) {
                GenerateUtils.pushBaiduLink("https://www.caimei365.com/info/detail-" + articlePo.getArticleId() + "-1.html");
            }
        }
        return ResponseJson.success(null);
    }

    @Override
    public ResponseJson<PageInfo<ShopArticleVo>> getShopArticleList(ShopArticleDto shopArticleDto, int pageNum, int pageSize) {
        if (null == shopArticleDto.getShopId()) {
            return ResponseJson.error("参数异常", null);
        }
        PageHelper.startPage(pageNum, pageSize);
        List<ShopArticleVo> articleList = articleMapper.getShopArticleList(shopArticleDto);
        PageInfo<ShopArticleVo> pageInfo = new PageInfo<>(articleList);
        return ResponseJson.success(pageInfo);
    }

    @Override
    public ResponseJson<Void> updateArticleStatus(ShopArticleDto shopArticleDto) {
        Integer articleId = shopArticleDto.getArticleId();
        Integer status = shopArticleDto.getStatus();
        if (null == articleId || null == status) {
            return ResponseJson.error("参数异常", null);
        }
        articleMapper.updateArticleStatus(articleId, status);
        // 更新文章索引
        commodityFeign.updateArticleIndex(articleId);
        return ResponseJson.success(null);
    }

    @Override
    public ResponseJson<List<ArticleTypeVo>> getArticleTypeList() {
        List<ArticleTypeVo> articleTypeList = articleMapper.getArticleTypeList("0");
        infoTypes(articleTypeList);
        return ResponseJson.success(articleTypeList);
    }

    public void infoTypes(List<ArticleTypeVo> articleTypes) {
        articleTypes.forEach(type -> {
            List<ArticleTypeVo> articleTypes1 = articleMapper.getArticleTypeList(type.getTypeId().toString());
            if (articleTypes1.size() > 0) {
                type.setArticleTypes(articleTypes1);
                infoTypes(articleTypes1);
            }
        });
    }

    @Override
    public ResponseJson<Void> deleteArticle(Integer articleId) {
        articleMapper.deleteArticle(articleId);
        // 更新文章索引
        commodityFeign.updateArticleIndex(articleId);
        return ResponseJson.success(null);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseJson<Void> saveBaikeProduct(BaikeProductDto baikeProductDto) {
        if (null == baikeProductDto.getShopId()) {
            return ResponseJson.error("参数异常，供应商id不能为空", null);
        }
        if (StringUtils.isEmpty(baikeProductDto.getName())) {
            return ResponseJson.error("参数异常，百科名称不能为空", null);
        }
        // 新增商品标识
        boolean newProductFlag = null == baikeProductDto.getProductId() || 0 == baikeProductDto.getProductId();
        BaikeProductPo baikeProductPo = new BaikeProductPo();
        BeanUtils.copyProperties(baikeProductDto, baikeProductPo);
        // 保存供应商百科商品
        if (newProductFlag) {
            baikeProductPo.setPublishSource(2);
            baikeProductPo.setAuditStatus(1);
            baikeProductPo.setBasePv(0);
            baikeProductPo.setActualPv(0);
            baikeProductPo.setOnlineStatus(1);
            baikeProductMapper.insertBaikeProduct(baikeProductPo);
        } else {
            baikeProductPo.setAuditStatus(1);
            baikeProductPo.setOnlineStatus(1);
            baikeProductMapper.updateBaikeProduct(baikeProductPo);
        }
        Integer productId = baikeProductPo.getProductId();
        // 删除原先的图片/参数/问题/视频文件数据
        if (!newProductFlag) {
            baikeProductMapper.deleteParamsByProductId(productId);
            baikeProductMapper.deleteImagesByProductId(productId);
            baikeProductMapper.deleteQuestionsByProductId(productId);
            baikeProductMapper.deleteFilesByProductId(productId);
        }
        // 保存参数
        JSONArray paramArray = JSONArray.parseArray(baikeProductDto.getParamList());
        if (null != paramArray) {
            paramArray.forEach(paramObject -> {
                Map<String, String> paramMap = (Map<String, String>) paramObject;
                String name = paramMap.get("name");
                String content = paramMap.get("content");
                if (StringUtils.isNotEmpty(name) && StringUtils.isNotEmpty(content)) {
                    baikeProductMapper.insertProductParam(name, content, productId);
                }
            });
        }
        // 保存认证图片
        List<String> authImageList = baikeProductDto.getAuthImageList();
        if (null != authImageList) {
            authImageList.forEach(authImage -> baikeProductMapper.insertProductImage(authImage, productId, 1));
        }
        // 保存展示图片
        List<String> displayImageList = baikeProductDto.getDisplayImageList();
        if (null != displayImageList) {
            displayImageList.forEach(displayImage -> baikeProductMapper.insertProductImage(displayImage, productId, 2));
        }
        // 保存问题
        JSONArray questionArray = JSONArray.parseArray(baikeProductDto.getQuestionList());
        if (null != questionArray) {
            questionArray.forEach(questionObject -> {
                Map<String, String> questionMap = (Map<String, String>) questionObject;
                String question = questionMap.get("question");
                String answer = questionMap.get("answer");
                if (StringUtils.isNotEmpty(question) && StringUtils.isNotEmpty(answer)) {
                    baikeProductMapper.insertProductQuestion(question, answer, productId);
                }
            });
        }
        // 保存视频文件
        JSONArray videoArray = JSONArray.parseArray(baikeProductDto.getVideoList());
        if (null != videoArray) {
            videoArray.forEach(videoObject -> {
                Map<String, String> videoMap = (Map<String, String>) videoObject;
                String fileTitle = videoMap.get("fileTitle");
                String fileName = videoMap.get("fileName");
                String ossName = videoMap.get("ossName");
                if (StringUtils.isNotEmpty(ossName)) {
                    baikeProductMapper.insertProductFile(fileTitle, fileName, ossName, productId);
                }
            });
        }
        return ResponseJson.success(null);
    }

    @Override
    public ResponseJson<BaikeProductFormVo> getBaikeProductById(Integer productId) {
        if (null == productId) {
            return ResponseJson.error("参数异常", null);
        }
        // 查询商品信息
        BaikeProductFormVo baikeProduct = baikeProductMapper.getBaikeProductById(productId);
        if (null == baikeProduct) {
            return ResponseJson.error("百科不存在", null);
        }
        // 参数列表
        List<BaikeProductParamPo> paramList = baikeProductMapper.getParamList(productId);
        baikeProduct.setParamList(paramList);
        // 产品/仪器认证图片列表
        List<String> authImageList = baikeProductMapper.getImageList(productId, 1);
        baikeProduct.setAuthImageList(authImageList);
        // 效果展示图片列表
        List<String> displayImageList = baikeProductMapper.getImageList(productId, 2);
        baikeProduct.setDisplayImageList(displayImageList);
        // 问题列表
        List<BaikeProductQuestionPo> questionList = baikeProductMapper.getQuestionList(productId);
        baikeProduct.setQuestionList(questionList);
        // 视频列表
        List<BaikeProductFilePo> videoList = baikeProductMapper.getVideoList(productId);
        videoList.forEach(video -> video.setFileUrl(OssUtil.getOssUrl(video.getOssName())));
        baikeProduct.setVideoList(videoList);
        return ResponseJson.success(baikeProduct);
    }


    @Override
    public ResponseJson<PageInfo<BaikeProductListVo>> getBaikeProductList(BaikeProductDto baikeProductDto, int pageNum, int pageSize) {
        if (null == baikeProductDto.getShopId() || null == baikeProductDto.getCommodityType()) {
            return ResponseJson.error("参数异常", null);
        }
        PageHelper.startPage(pageNum, pageSize);
        List<BaikeProductListVo> baikeProductList = baikeProductMapper.getBaikeProductList(baikeProductDto);
        PageInfo<BaikeProductListVo> pageInfo = new PageInfo<>(baikeProductList);
        return ResponseJson.success(pageInfo);
    }

    @Override
    public ResponseJson<Void> updateBaikeProductStatus(BaikeProductDto baikeProductDto) {
        Integer productId = baikeProductDto.getProductId();
        Integer status = baikeProductDto.getStatus();
        if (null == productId || null == status) {
            return ResponseJson.error("参数异常", null);
        }
        baikeProductMapper.updateBaikeProductStatus(productId, status);
        return ResponseJson.success(null);
    }

    @Override
    public ResponseJson<List<BaikeProductTypeVo>> getBaikeProductTypeList(Integer commodityType) {
        List<BaikeProductTypeVo> baikeProductTypeList = baikeProductMapper.getBaikeProductTypeList(commodityType);
        return ResponseJson.success(baikeProductTypeList);
    }

    @Override
    public ResponseJson<Void> deleteBaikeProduct(Integer productId) {
        baikeProductMapper.deleteBaikeProduct(productId);
        return ResponseJson.success(null);
    }

    @Override
    public ResponseJson<String> getSensitiveWords(Integer checkPoint) {
        String sensitiveWords = baseMapper.getSensitiveWords(checkPoint);
        return ResponseJson.success(sensitiveWords);
    }

    @Override
    public ResponseJson<Map<String, Object>> getBaikeRecommendForm(Integer productId) {
        HashMap<String, Object> map = new HashMap<>();
        BaikeProductFormVo baikeProduct = baikeProductMapper.getBaikeProductById(productId);
        Integer productTypeId = baikeProduct.getTypeId();
        Integer recommendType = baikeProduct.getRecommendType();
        Integer shopId = baikeProduct.getShopId();
        // 手动推荐列表
        List<BaikeRecommendVo> manualRecommendList = baikeProductMapper.getManualRecommendList(productId);
        // 自动推荐列表
        List<BaikeRecommendVo> autoRecommendList = baikeProductMapper.getAutoRecommendList(shopId, productTypeId, productId);
        autoRecommendList.forEach(authRecommend -> {
            authRecommend.setProductId(productId);
        });
        map.put("manualRecommendList", manualRecommendList);
        map.put("autoRecommendList", autoRecommendList);
        map.put("recommendType", recommendType);
        return ResponseJson.success(map);
    }

    @Override
    public ResponseJson<Void> saveRecommendType(BaikeProductDto baikeProductDto) {
        baikeProductMapper.updateRecommendType(baikeProductDto.getProductId(), baikeProductDto.getRecommendType());
        return ResponseJson.success(null);
    }

    @Override
    public ResponseJson<PageInfo<BaikeProductListVo>> getAddRecommendList(Integer productId, Integer typeId, String name, int pageNum, int pageSize) {
        BaikeProductDto baikeProductDto = new BaikeProductDto();
        baikeProductDto.setTypeId(typeId);
        baikeProductDto.setName(name);
        BaikeProductFormVo baikeProduct = baikeProductMapper.getBaikeProductById(productId);
        baikeProductDto.setShopId(baikeProduct.getShopId());
        // 已推荐产品id，以，隔开
        String productIds = baikeProductMapper.getRecommendProductIds(productId);
        productIds += "".equals(productIds) ? productId : "," + productId;
        baikeProductDto.setProductIds(productIds);
        baikeProductDto.setAuditStatus(2);
        PageHelper.startPage(pageNum, pageSize);
        List<BaikeProductListVo> addRecommendList = baikeProductMapper.getBaikeProductList(baikeProductDto);
        PageInfo<BaikeProductListVo> pageInfo = new PageInfo<>(addRecommendList);
        return ResponseJson.success(pageInfo);
    }

    @Override
    public ResponseJson<Void> addBaikeRecommend(BaikeProductDto baikeProduct) {
        String baikeRecommendList = baikeProduct.getBaikeRecommendList();
        List<BaikeRecommendVo> recommendList = JSONObject.parseArray(baikeRecommendList, BaikeRecommendVo.class);
        recommendList.forEach(baikeRecommend -> {
            baikeRecommend.setProductId(baikeProduct.getProductId());
            baikeProductMapper.insertRecommend(baikeRecommend);
        });
        return ResponseJson.success(null);
    }

    @Override
    public ResponseJson<Void> deleBaikeRecommend(Integer recommendId) {
        baikeProductMapper.deleteBaikeRecommend(recommendId);
        return ResponseJson.success(null);
    }

    @Override
    public ResponseJson<Void> saveRecommendSort(BaikeRecommendVo baikeRecommend) {
        baikeProductMapper.updateRecommendSort(baikeRecommend);
        return ResponseJson.success(null);
    }

    /**
     * 供应商营销数据看板
     *
     * @param type
     * @param marketReportId
     * @param shopId
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public ResponseJson<Map<String, Object>> preview(Integer type, Integer marketReportId, Integer shopId, String startTime, String endTime) {
        Map<String, Object> map = new HashMap<>();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM");
        SimpleDateFormat formatDate = new SimpleDateFormat("yyyy-MM-dd");
        try {
            if (StringUtils.isNotBlank(startTime)) {
                startTime = dateFormat.format(dateFormat.parse(startTime));
            }
            if (StringUtils.isNotBlank(endTime)) {
                endTime = dateFormat.format(dateFormat.parse(endTime));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        // name集合
        ArrayList<String> name = new ArrayList<>();
        // value 集合
        ArrayList<Integer> value = new ArrayList<>();
        // 备份（第二value）集合
        ArrayList<Integer> backupsValues = new ArrayList<>();
        // 唯一name集合
        Set<String> set = new HashSet<>();
        // 获取报表对应供应商数据
        CmMarketShopVo shopInfo = shopMapper.getShopInfoById(shopId, type, marketReportId);
        List<Integer> marketReportIds = shopMapper.getMarketReportIds(shopInfo.getId(), type, marketReportId, startTime, endTime);
        log.info("marketReportIds====" + marketReportIds);
        // 获取阶段描述
        List<StageVo> summarize = new ArrayList<>();
        List<StageVo> summarize1 = new ArrayList<>();
        List<StageVo> summarize2 = new ArrayList<>();
        if (null != marketReportIds && marketReportIds.size() > 0) {
            List<StageVo> marketStage = shopMapper.getMarketStage(marketReportIds);
            if (null != marketStage && marketStage.size() > 0) {
                for (StageVo stage : marketStage) {
                    try {
                        stage.setAddTime(dateFormat.format(dateFormat.parse(stage.getReportDate())));
                        if ("1".equals(stage.getStage())) {
                            summarize.add(stage);
                        }
                        if ("2".equals(stage.getStage())) {
                            summarize1.add(stage);
                        }
                        if ("3".equals(stage.getStage())) {
                            summarize2.add(stage);
                        }
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
        map.put("summarize", summarize);
        map.put("summarize1", summarize1);
        map.put("summarize2", summarize2);
        // 获取漏斗模型数据
        if (null != marketReportIds && marketReportIds.size() > 0) {
            List<FunnelModelVo> marketFunnelModel = shopMapper.getMarketFunnelModel(marketReportIds);
            int impressions = 0, hits = 0, visits = 0, consultation = 0, report = 0;
            if (null != marketFunnelModel && marketFunnelModel.size() > 0) {
                for (FunnelModelVo funnelModel : marketFunnelModel) {
                    impressions += funnelModel.getImpressions();
                    hits += funnelModel.getHits();
                    visits += funnelModel.getVisits();
                    consultation += funnelModel.getConsultation();
                    report += funnelModel.getReport();
                }
            }
            try {
                if (StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime)) {
                    shopInfo.setChartTitle("(" + startTime.replaceAll("-", "/") + "-" + endTime.replaceAll("-", "/") + ")");
                } else if (1 == type && null != marketReportId) {
                    shopInfo.setChartTitle("(" + dateFormat.format(dateFormat.parse(shopInfo.getReportDate())).replaceAll("-", "/") + ")");
                } else {
                    shopInfo.setChartTitle("(" + (StringUtils.isNotBlank(shopInfo.getEndDate()) ? dateFormat.format(dateFormat.parse(shopInfo.getEndDate())).replaceAll("-", "/") : "") + ")");
                }
            } catch (ParseException e) {
                e.printStackTrace();
            }
            shopInfo.setImpressions(impressions);
            shopInfo.setHits(hits);
            shopInfo.setVisits(visits);
            shopInfo.setConsultation(consultation);
            shopInfo.setReport(report);
        }
        map.put("shopInfo", shopInfo);
        // 获取推文数据
        List<ArticleReadVolumeVo> articleReadVolumeList = new ArrayList<>();
        if (null != marketReportIds && marketReportIds.size() > 0) {
            List<ArticleReadVolumeVo> marketArticleReadVolume = shopMapper.getMarketArticleReadVolume(marketReportIds);
            if (null != marketArticleReadVolume && marketArticleReadVolume.size() > 0) {
                for (ArticleReadVolumeVo articleReadVolume : marketArticleReadVolume) {
                    try {
                        articleReadVolume.setFirstTime(formatDate.format(formatDate.parse(articleReadVolume.getFirstTime())));
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                    if (!set.contains(articleReadVolume.getTitle())) {
                        set.add(articleReadVolume.getTitle());
                        articleReadVolumeList.add(articleReadVolume);
                    } else {
                        // 将有重复的推文数据 总阅读量  总展现量相加为一组数据
                        for (ArticleReadVolumeVo readVolume : articleReadVolumeList) {
                            if (articleReadVolume.getTitle().equals(readVolume.getTitle())) {
                                readVolume.setWechatNumber(MathUtil.add(readVolume.getWechatNumber(), articleReadVolume.getWechatNumber()).intValue());
                                readVolume.setTouchNumber(MathUtil.add(readVolume.getTouchNumber(), articleReadVolume.getTouchNumber()).intValue());
                            }
                        }
                    }
                }
            }
        }
        map.put("volumeList", articleReadVolumeList);
        // 公众号推文数据
        ReturnDataVo articleReturnData = new ReturnDataVo();
        if (null != marketReportIds && marketReportIds.size() > 0) {
            if (null != articleReadVolumeList && articleReadVolumeList.size() > 0) {
                name = new ArrayList<>(articleReadVolumeList.stream().map(ArticleReadVolumeVo::getTitleReferred).collect(Collectors.toList()));
                articleReturnData.setNames(name);
                value = new ArrayList<>(articleReadVolumeList.stream().map(ArticleReadVolumeVo::getWechatNumber).collect(Collectors.toList()));
                articleReturnData.setValues(value);
                backupsValues = new ArrayList<>(articleReadVolumeList.stream().map(ArticleReadVolumeVo::getTouchNumber).collect(Collectors.toList()));
                articleReturnData.setBackupsValues(backupsValues);
            }
        }
        map.put("wechats", articleReturnData);
        // 获取展现量数据
        name = new ArrayList<>();
        value = new ArrayList<>();
        int generalNumber = 0, microBlog = 0, redBookNumber = 0, otherNumber = 0, pointSingNumber = 0;
        if (null != marketReportIds && marketReportIds.size() > 0) {
            List<ImpressionsVo> marketImpressions = shopMapper.getMarketImpressions(marketReportIds);
            if (null != marketImpressions && marketImpressions.size() > 0) {
                for (ImpressionsVo imp : marketImpressions) {
                    generalNumber += imp.getGeneralNumber();
                    microBlog += imp.getMicroBlog();
                    redBookNumber += imp.getRedBookNumber();
                    pointSingNumber += imp.getPointSingNumber();
                    otherNumber += imp.getOtherNumber();
                }
            }
        }
        ReturnDataVo returnData = new ReturnDataVo();
        name.add("公众号");
        name.add("微博");
        name.add("小红书");
        name.add("一点号");
        name.add("其它自媒体渠道");
        returnData.setNames(name);
        value.add(generalNumber);
        value.add(microBlog);
        value.add(redBookNumber);
        value.add(pointSingNumber);
        value.add(otherNumber);
        returnData.setValues(value);
        map.put("intention", returnData);
        // 获取点击量数据
        name = new ArrayList<>();
        value = new ArrayList<>();
        generalNumber = 0;
        microBlog = 0;
        redBookNumber = 0;
        otherNumber = 0;
        pointSingNumber = 0;
        if (null != marketReportIds && marketReportIds.size() > 0) {
            List<ReadVolumeVo> marketReadVolume = shopMapper.getMarketReadVolume(marketReportIds);
            if (null != marketReadVolume && marketReadVolume.size() > 0) {
                for (ReadVolumeVo readVolume : marketReadVolume) {
                    generalNumber += readVolume.getGeneralNumber();
                    microBlog += readVolume.getMicroBlog();
                    redBookNumber += readVolume.getRedBookNumber();
                    pointSingNumber += readVolume.getPointSingNumber();
                    otherNumber += readVolume.getOtherNumber();
                }
            }
        }
        ReturnDataVo readVolumeReturnData = new ReturnDataVo();
        name.add("公众号");
        name.add("微博");
        name.add("小红书");
        name.add("一点号");
        name.add("其它自媒体渠道");
        readVolumeReturnData.setNames(name);
        value.add(generalNumber);
        value.add(microBlog);
        value.add(redBookNumber);
        value.add(pointSingNumber);
        value.add(otherNumber);
        readVolumeReturnData.setValues(value);
        map.put("allVisits", readVolumeReturnData);
        // 获取SEO汇总数据
        List<SummaryVo> summaryList = new ArrayList<>();
        set = new HashSet<>();
        if (null != marketReportIds && marketReportIds.size() > 0) {
            List<SummaryVo> marketSummary = shopMapper.getMarketSummary(marketReportIds);
            List<ReturnDataVo> returnDataList = new ArrayList<>();
            if (null != marketSummary && marketSummary.size() > 0) {
                for (SummaryVo summary : marketSummary) {
                    if (!set.contains(summary.getKeyword())) {
                        set.add(summary.getKeyword());
                        summaryList.add(summary);
                    } else {
                        for (SummaryVo summary1 : summaryList) {
                            if (summary.getKeyword().equals(summary1.getKeyword())) {
                                summary1.setSearchVolume(MathUtil.add(summary1.getSearchVolume(), summary.getSearchVolume()).intValue());
                            }
                        }
                    }
                }
            }
            if (null != summaryList && summaryList.size() > 0) {
                for (SummaryVo summary : summaryList) {
                    ReturnDataVo summaryReturnData = new ReturnDataVo();
                    summaryReturnData.setName(summary.getKeyword());
                    summaryReturnData.setValue(summary.getSearchVolume());
                    summaryReturnData.setSeo(summary.getSeoRanking());
                    returnDataList.add(summaryReturnData);
                }
            }
        }
        map.put("keywordList", summaryList);
        // 获取访问量数据
        name = new ArrayList<>();
        value = new ArrayList<>();
        int clubNumber = 0, touristNumber = 0;
        if (null != marketReportIds && marketReportIds.size() > 0) {
            List<VisitsVo> marketVisits = shopMapper.getMarketVisits(marketReportIds);
            if (null != marketVisits && marketVisits.size() > 0) {
                for (VisitsVo vis : marketVisits) {
                    clubNumber += vis.getClubNumber();
                    touristNumber += vis.getTouristNumber();
                }
            }
        }
        ReturnDataVo visitsReturnData = new ReturnDataVo();
        name.add("机构访问量");
        name.add("游客访问量");
        visitsReturnData.setNames(name);
        value.add(clubNumber);
        value.add(touristNumber);
        visitsReturnData.setValues(value);
        map.put("stationVisits", visitsReturnData);
        // 获取访客来源分布数据
        int search = 0, general = 0, proMicroBlog = 0, directAccess = 0;
        if (null != marketReportIds && marketReportIds.size() > 0) {
            List<ProportionVo> marketProportion = shopMapper.getMarketProportion(marketReportIds);
            if (null != marketProportion && marketProportion.size() > 0) {
                for (ProportionVo proportion : marketProportion) {
                    search += proportion.getSearch();
                    general += proportion.getGeneral();
                    proMicroBlog += proportion.getMicroBlog();
                    directAccess += proportion.getDirectAccess();
                }
            }
        }
        List<ReturnDataVo> proportionDataList = new ArrayList<>();
        ReturnDataVo proportionReturnData = new ReturnDataVo();
        proportionReturnData.setName("搜索引擎");
        proportionReturnData.setValue(search);
        proportionDataList.add(proportionReturnData);
        proportionReturnData = new ReturnDataVo();
        proportionReturnData.setName("微信平台");
        proportionReturnData.setValue(general);
        proportionDataList.add(proportionReturnData);
        proportionReturnData = new ReturnDataVo();
        proportionReturnData.setName("微博");
        proportionReturnData.setValue(proMicroBlog);
        proportionDataList.add(proportionReturnData);
        proportionReturnData = new ReturnDataVo();
        proportionReturnData.setName("直接访问");
        proportionReturnData.setValue(directAccess);
        proportionDataList.add(proportionReturnData);
        map.put("proportion", proportionDataList);
        // 获取页面平均访问时长数据
        set = new HashSet<>();
        List<PageDurationVo> pageDurationList = new ArrayList<>();
        if (null != marketReportIds && marketReportIds.size() > 0) {
            List<PageDurationVo> marketPageDuration = shopMapper.getMarketPageDuration(marketReportIds);
            if (null != marketPageDuration && marketPageDuration.size() > 0) {
                for (PageDurationVo pageDuration : marketPageDuration) {
                    if (!set.contains(pageDuration.getPageLabel())) {
                        set.add(pageDuration.getPageLabel());
                        pageDurationList.add(pageDuration);
                    } else {
                        for (PageDurationVo pageDuration1 : pageDurationList) {
                            if (pageDuration.getPageLabel().equals(pageDuration1.getPageLabel())) {
                                pageDuration1.setAccessDuration(MathUtil.add(pageDuration1.getAccessDuration(), pageDuration.getAccessDuration()).intValue());
                            }
                        }
                    }
                }
            }
        }
        ReturnDataVo pageDurationReturnData = new ReturnDataVo();
        name = new ArrayList<>(set);
        pageDurationReturnData.setNames(name);
        value = new ArrayList<>(pageDurationList.stream().map(PageDurationVo::getAccessDuration).collect(Collectors.toList()));
        pageDurationReturnData.setValues(value);
        map.put("visitTimes", pageDurationReturnData);
        // 获取广告图点击量数据
        List<AdvertHitsVo> marketAdvertHits = new ArrayList<>();
        if (null != marketReportIds && marketReportIds.size() > 0) {
            marketAdvertHits = shopMapper.getMarketAdvertHits(marketReportIds);
        }
        map.put("bannerList", marketAdvertHits);
        // 获取咨询数量数据
        name = new ArrayList<>();
        value = new ArrayList<>();
        if (null != marketReportIds && marketReportIds.size() > 0) {
            List<CmMarketRemarkVo> marketRemark = shopMapper.getMarketRemark(marketReportIds);
            if (null != marketRemark && marketRemark.size() > 0) {
                for (CmMarketRemarkVo remark : marketRemark) {
                    try {
                        name.add(dateFormat.format(dateFormat.parse(remark.getAddTime())));
                        value.add(remark.getRemarkNumber());
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
        ReturnDataVo remarkReturnData = new ReturnDataVo();
        remarkReturnData.setNames(name);
        remarkReturnData.setValues(value);
        map.put("leadUserMonthly", remarkReturnData);
        // 供应商相关标签
        List<String> shopKeyword = shopMapper.getShopKeyword(shopInfo.getShopId());

        List<ReturnDataVo> regionMap = new ArrayList<>();
        // 访问供应商用户对应身份
        List<ReturnEntityVo> userIdentity = new ArrayList<>();
        // 访问供应商用户对应机构类型
        List<ReturnEntityVo> clubType = new ArrayList<>();
        if (1 == type) {
            // 预览
            if (StringUtils.isBlank(startTime)) {
                startTime = monthFirstDay(shopInfo.getReportDate());
            }
            if (StringUtils.isBlank(endTime)) {
                endTime = monthLastDay(shopInfo.getReportDate());
            }
        } else {
            // 供应商主页进入
            if (StringUtils.isBlank(startTime)) {
                startTime = shopInfo.getStartDate();
            }
            if (StringUtils.isBlank(endTime)) {
                endTime = shopInfo.getEndDate();
            }

        }
        log.info("startTime====" + startTime);
        log.info("endTime====" + endTime);
        log.info("shopKeyword====" + shopKeyword);
        if (null != shopKeyword && shopKeyword.size() > 0) {
            // 访问供应商用户对于所在地区
            List<String> region = shopMapper.getRegion(startTime, endTime, shopKeyword);
            region.removeIf(re -> re.length() < 2);
            Set<String> province = new HashSet<>();
            String com = "";
            for (String str : region) {
                String trim = str.trim();
                com = trim.substring(0, 2);
                province.add(com);
            }
            for (String str : province) {
                ReturnDataVo rData = new ReturnDataVo();
                rData.setName(str);
                List<String> collect = region.stream().filter(pro -> pro.substring(0, 3).contains(str)).collect(Collectors.toList());
                rData.setValue(collect.size());
                regionMap.add(rData);
            }
            set = new HashSet<>();
            List<ReturnEntityVo> userIdentity1 = shopMapper.getUserIdentity(startTime, endTime, shopKeyword);
            if (null != userIdentity1 && userIdentity1.size() > 0) {
                userIdentity1.removeIf(user -> StringUtils.isBlank(user.getName()));
                // 所有值之和
                AtomicInteger maxUserValue = new AtomicInteger(0);
                for (ReturnEntityVo returnEntityVo : userIdentity1) {
                    // maxUserValue.set(MathUtil.add(Integer.parseInt(returnEntityVo.getValue()),maxUserValue.get()).intValue());
                    if (!set.contains(returnEntityVo.getName())) {
                        set.add(returnEntityVo.getName());
                        userIdentity.add(returnEntityVo);
                    } else {
                        for (ReturnEntityVo returnUserIdentity : userIdentity) {
                            returnUserIdentity.setValue(MathUtil.add(returnUserIdentity.getValue(), returnEntityVo.getValue()).toString());
                        }
                    }
                }
                userIdentity.forEach(user -> maxUserValue.set(MathUtil.add(Integer.parseInt(user.getValue()), maxUserValue.get()).intValue()));
                userIdentity.forEach(returnUserIdentity -> {
                    log.info("returnUserIdentity.getValue()===" + returnUserIdentity.getValue());
                    log.info("maxUserValue.get()===" + maxUserValue.get());
                    String values = MathUtil.mul(MathUtil.div(returnUserIdentity.getValue(), maxUserValue.get(), 2), 100).toString();
                    log.info("values===" + values);
                    returnUserIdentity.setValue(values);
                });
            }
            set = new HashSet<>();
            List<ReturnEntityVo> clubType1 = shopMapper.getClubType(startTime, endTime, shopKeyword);
            if (null != clubType1 && clubType1.size() > 0) {
                clubType1.removeIf(club -> StringUtils.isBlank(club.getName()));
                // 所有值之和
                AtomicInteger maxClubTypeValue = new AtomicInteger(0);
                for (ReturnEntityVo returnEntityVo : clubType1) {
                    // maxClubTypeValue.set(MathUtil.add(Integer.parseInt(returnEntityVo.getValue()), maxClubTypeValue.get()).intValue());
                    if (!set.contains(returnEntityVo.getName())) {
                        set.add(returnEntityVo.getName());
                        clubType.add(returnEntityVo);
                    } else {
                        for (ReturnEntityVo returnUserIdentity : clubType) {
                            returnUserIdentity.setValue(MathUtil.add(returnUserIdentity.getValue(), returnEntityVo.getValue()).toString());
                        }
                    }
                }
                clubType.forEach(club -> maxClubTypeValue.set(MathUtil.add(Integer.parseInt(club.getValue()), maxClubTypeValue.get()).intValue()));
                clubType.forEach(returnUserIdentity -> {
                    log.info("returnUserIdentity.getValue()===" + returnUserIdentity.getValue());
                    log.info("maxClubTypeValue.get()===" + maxClubTypeValue.get());
                    String values = MathUtil.mul(MathUtil.div(returnUserIdentity.getValue(), maxClubTypeValue.get(), 2), 100).toString();
                    log.info("values===" + values);
                    returnUserIdentity.setValue(values);
                });
            }
        }
        map.put("channels", regionMap);
        map.put("userIdentity", userIdentity);
        map.put("urbanize", clubType);
        // 是否有数据
        if (null == marketReportIds || marketReportIds.size() <= 0) {
            map.put("code", -1);
        } else {
            map.put("code", 0);
        }
        // 时间筛选
        List<String> marketReportDataList = new ArrayList<>();
        if (null != shopInfo) {
            marketReportDataList = shopMapper.getMarketReportDataList(shopInfo.getShopId());
            List<String> yearMonth = monthList(marketReportDataList);
            map.put("yearMonth", yearMonth);
        } else {
            List<String> yearMonth = monthList(marketReportDataList);
            map.put("yearMonth", yearMonth);
        }
        return ResponseJson.success(map);
    }

    /**
     * 获取该月份首日日期
     *
     * @param reportDate 2023-12
     * @return
     */
    public String monthFirstDay(String reportDate) {
        String[] split = reportDate.split("-");
        Integer year = Integer.parseInt(split[0]);
        Integer month = Integer.parseInt(split[1]);

        LocalDate firstDayOfMonth = LocalDate.of(year, month, 1);
        return firstDayOfMonth.toString();
    }

    /**
     * 获取该月份最后一天日期
     *
     * @param reportDate 2023-12
     * @return
     */
    public String monthLastDay(String reportDate) {
        String[] split = reportDate.split("-");
        Integer year = Integer.parseInt(split[0]);
        Integer month = Integer.parseInt(split[1]);

        LocalDate firstDayOfMonth = LocalDate.of(year, month, 1);
        LocalDate lastDayOfMonth = firstDayOfMonth.with(TemporalAdjusters.lastDayOfMonth());
        return lastDayOfMonth.toString();
    }

    /**
     * 不包含的月份
     * todo 暂定5年
     *
     * @param marketReportDataList
     * @return
     */
    public List<String> monthList(List<String> marketReportDataList) {
        Integer[] yearList = new Integer[]{2023, 2024, 2025, 2026, 2027};
        ArrayList<String> yearMonth = new ArrayList<>();
        for (Integer year : yearList) {
            List<YearMonth> collect = Stream.iterate(YearMonth.of(year, 1), month -> month.plusMonths(1)).limit(12).collect(Collectors.toList());
            String strip = StringUtils.strip(collect.toString(), "[]");
            String[] split = strip.split(",");
            for (String s : split) {
                yearMonth.add(s.replaceAll("-", "").trim());
            }
        }
        if (null != marketReportDataList && marketReportDataList.size() > 0) {
            marketReportDataList.replaceAll(market -> market.replaceAll("-", ""));
        }
        yearMonth.removeIf(marketReportDataList::contains);
        return yearMonth;
    }
}
